//=============================================================================
//                                                            FFCal_v03_mod_v1.5.mq4
//                                            Copyright  2006, Derk Wehler
// Written in cooperation with:                 http://www.forexfactory.com
//
//
// This "indicator" calls DLLs to fetch a special XML file from the 
// ForexFactory web site.  It then parses it and writes it out as a .CSV 
// file, which it places in the folder: experts/files so that IsNewsTime() 
// can use that file to tell if it is near announcement time.
//
// It does this once when it starts up and once per hour in case there have 
// been any updates to the annoucement calendar.
//
// In order for this to work correctly and synchronize with your broker's 
// server time, you must enter the proper value for "TimeZone".  This is 
// entered relative to GMT.  For example, my broker's server time is GMT+1, 
// so I have the default set to 1.  If your broker uses GMT+3, enter 3.
// If your broker uses GMT-2, then enter -2.
//
//
//
//=============================================================================
//                             OTHER CREDIT DUE
//      (For GrebWeb and LogUtils functionality (see end of this flie)
//
//  2/14/2007:  Robert Hill added code for using text objects instead
//              of Comment() for easier reading
//
//  2/25/2007:  Paul Hampton-Smith for his TimeZone DLL code
//=============================================================================
//+---------------------------------------------------------------------------+
//|                                                               WinInet.mqh |
//|                                                        Paul Hampton-Smith |
//|                                                        paul1000@pobox.com |
//|                                                                           |
//| This include has been adapted from the groundbreaking GrabWeb script      |
//| developed by Adbi that showed me how to import functions from wininet.dll |
//|---------------------------------------------------------------------------|
//|                                                               grabweb.mq4 |
//|                                                    Copyright  2006, Abhi |
//|                                         http://www.megadelfi.com/experts/ |
//|                                     E-mail: grabwebexpert{Q)megadelfi.com |
//|                                fix my e-mail address before mailing me ;) |
//+---------------------------------------------------------------------------+
/*
		IDEAS FOR ADDITIONS:
		
*   A text lable of what the news was and when it was, e.g
    "USD CPI m/m @ [broker time here]" at the top or bottom within the shaded area.
    
*   Label the number of pips it went in its initial direction and how much it 
	retraced...of course you'd have to come up with a convention for measuring 
	movement.  Or you could label what price was just prior to news then labels 
	the next two price exteremes.  I think it would be nice to have the numbers 
	right under/over the candle that produced the extremes for whatever timeframe 
	you are in.

*   Label the volume count for the candle that included the news in it...whatever 
	timeframe your in.  So I could switch from 1 min. to 5 min. to 15 min. 
	timeframes and get different vol. counts for the bar that included the news.
	
*/

#property copyright "Copyright  2006, Derk Wehler"
#property link      "http://www.forexfactory.com"

#property indicator_chart_window
#property indicator_buffers 8

#define TITLE		0
#define COUNTRY	1
#define DATE		2
#define TIME		3
#define IMPACT		4
#define FORECAST	5
#define PREVIOUS	6

//====================================================================================
//====================================================================================
//====================================================================================
//====================================================================================
//====================================================================================
//====================================================================================
//====================================================================================

extern bool    Text              = true;
extern bool    TextOutput        = true;
extern bool 	IncludeHigh 		= true;
extern bool 	IncludeMedium 		= true;
extern bool 	IncludeLow 			= true;
extern bool 	IncludeSpeaks 		= true; 	// news items with "Speaks" in them have different characteristics
extern bool    bRecordTickLog = true; // .csv file of Bids & Asks
extern bool    bRecordScreenShot = true; // screenshot nTimeWindowMinutes after event

extern int		Alert1MinsBefore	= 400;//-1;		// Set to -1 for no Alert

extern bool 	EnableLogging 		= false; 	// Perhaps remove this from externs once its working well
extern bool		ShowNextTwoEvents	= true;
extern int 		TxtSize 			= 10;
extern color 	TxtColorTitle 		= LightGray;
extern color 	TxtColorNews 		= DeepSkyBlue;
extern color 	TxtColorImpact 		= Red;
extern color 	TxtColorPrevious 	= Orange;
extern color 	TxtColorForecast 	= Lime;

string         comment_str1="-";
string         comment_str2="-";
int		      DebugLevel = 2;


double 	ExtMapBuffer0[];	// Contains (minutes until) each news event
double 	ExtMapBuffer1[];	// Contains only most recent and next news event ([0] & [1])
double 	ExtMapBuffer2[];	// Contains impact value for most recent and next news event
double 	ExtMapBuffer3[];  // newsIdx
double 	ExtMapBuffer4[];  // NewsDateTime
double 	ExtMapBuffer5[];  // Impact
double 	ExtMapBuffer6[];  // Previous
double 	ExtMapBuffer7[];  // Forecast

//fxfishermann -->http://64.22.88.90/forexcalendar/forecast.aspx
string	    sUrl = "http://www.forexfactory.com/weekly_calendar.xml";
int 	       xmlHandle;
int 	       logHandle = -1;
int 	       BoEvent, finalend, end, i;
int		    begin;
string 	    mainData[100][7];
int 	       minsTillNews;
string 	    sData, csvoutput;
string	    commentStr;
int		    tmpMins;
int		    idxOfNext;
int		    dispMinutes[2];
string 	    dispTitle[2], 
		       dispCountry[2], 
		       dispImpact[2], 
		       dispForecast[2], 
		       dispPrevious[2];
		       
string	    dispSeparator1 = "===================";

static bool  FirstRun = true;
static int	 PrevMinute = -1;


int init()
{

   create_globals();
   
	Print("In Init()...\n");
	
	SetIndexStyle(0, DRAW_NONE);
	SetIndexBuffer(0, ExtMapBuffer0);
   SetIndexEmptyValue(0,0.0);
	SetIndexStyle(1, DRAW_NONE);
	SetIndexBuffer(1, ExtMapBuffer1);
   SetIndexEmptyValue(1,0.0);
	SetIndexStyle(2, DRAW_NONE);
	SetIndexBuffer(2, ExtMapBuffer2);
   SetIndexEmptyValue(2,0.0);
	SetIndexStyle(3, DRAW_NONE);
	SetIndexBuffer(3, ExtMapBuffer3);
   SetIndexEmptyValue(3,0.0);	
   
	SetIndexStyle(4, DRAW_NONE);
	SetIndexBuffer(4, ExtMapBuffer4);
   SetIndexEmptyValue(4,0.0);	
	SetIndexStyle(5, DRAW_NONE);
	SetIndexBuffer(5, ExtMapBuffer5);
   SetIndexEmptyValue(5,0.0);	
	SetIndexStyle(6, DRAW_NONE);
	SetIndexBuffer(6, ExtMapBuffer6);
   SetIndexEmptyValue(6,0.0);	
	SetIndexStyle(7, DRAW_NONE);
	SetIndexBuffer(7, ExtMapBuffer7);
   SetIndexEmptyValue(7,0.0);	

	
   SetIndexEmptyValue(3,0.0);
	IndicatorShortName("FFCal");
	SetIndexLabel(0, "MinsBeforeNews");
	SetIndexLabel(1, "RecentandNext"); 
	SetIndexLabel(2, "ImpactRecentandNext"); 
   SetIndexLabel(3, "newsIdx"); 	
   SetIndexLabel(4, "NewsDateTime"); 	
   SetIndexLabel(5, "Impact"); 	
   SetIndexLabel(6, "Previous"); 	
   SetIndexLabel(7, "Forecast"); 	            

	return(0);
}


int deinit()
{
  
   deinit_globals();
   
	ObjectDelete("Sponsor"); 
	ObjectDelete("Minutes"); 
	ObjectDelete("Impact");
	ObjectDelete("Previous"); 
	ObjectDelete("Forecast");
	ObjectDelete("Separator1"); 
	ObjectDelete("Separator2");

	ObjectDelete("Sponsor2"); 
	ObjectDelete("Minutes2"); 
	ObjectDelete("Impact2");
	ObjectDelete("Previous2"); 
	ObjectDelete("Forecast2");

	return(0);
}


int start()
{
	int 		newsIdx = 0;
	int 		nextNewsIdx = -1;
	int 		next;
	string 	myEvent;
	bool 		skip;
	datetime newsTime;
	
	init_global1();
	init_global2();
	
	// If we are not logging, then do not output debug statements either
	if (!EnableLogging)
		DebugLevel = 0;
				
	commentStr = "Forex Factory Calendar";
	string xmlFileName = Month() + "-" + Day() + "-" + Year() + "-" + "FFCal.xml";
	Print("STARTED");
	
	// Refresh FF calendar on startup, and once at the top of each hour
	if (FirstRun || Minute() == 0)
	{
		FirstRun = false;

		if (DebugLevel > 1)
			Print("sUrl == ", sUrl);
		
		if (DebugLevel > 0)
			Print("Grabbing Web, url = ", sUrl);
	
		// THIS CALL WAS DONATED BY PAUL TO HELP FIX THE RESOURCE ERROR
		GrabWeb(sUrl, sData);

		if (DebugLevel > 0)
		{
			Print("Opening XML file...\n");
			Print(sData);
		}

		// Write the contents of the ForexFactory page to an .htm file
		xmlHandle = FileOpen(xmlFileName, FILE_BIN|FILE_READ|FILE_WRITE);
		if (xmlHandle < 1)
		{
			if (DebugLevel > 0)
				Print("Can\'t open new xml file, the last error is ", GetLastError());
			return(false);
		}
		FileWriteString(xmlHandle, sData, StringLen(sData));
		FileClose(xmlHandle);

		if (DebugLevel > 0)
			Print("Wrote XML file...\n");

		// THIS BLOCK OF CODE DONATED BY WALLY TO FIX THE RESOURCE ERROR
		//--- Look for the end XML tag to ensure that a complete page was downloaded ---//
		end = StringFind(sData, "</weeklyevents>", 0);

		if (end <= 0)
		{
			Alert("GetNewsFF Error - Web page download was not complete!");
			return(false);
		}
		//-------------------------------------------------------------------------------//
	}

	// = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
	// Perform remaining checks once per minute
	if (Minute() == PrevMinute)
		return (true);
	PrevMinute = Minute();
	Print("NEW MINUTE...");

	// Open the log file (will not open if logging is turned off)
	OpenLog("FFCal");

	// Init the buffer array to zero just in case
	ArrayInitialize(ExtMapBuffer0, 0);
	ArrayInitialize(ExtMapBuffer1, 0);
	ArrayInitialize(ExtMapBuffer2, 0);
	ArrayInitialize(ExtMapBuffer3, 0);
	ArrayInitialize(ExtMapBuffer4, 0);
	ArrayInitialize(ExtMapBuffer5, 0);
	ArrayInitialize(ExtMapBuffer6, 0);
	ArrayInitialize(ExtMapBuffer7, 0);				
	
	// Open the XML file
	xmlHandle = FileOpen(xmlFileName, FILE_BIN|FILE_READ);
	if (xmlHandle < 0)
	{
		Print("Can\'t open xml file: ", xmlFileName, ".  The last error is ", GetLastError());
		return(false);
	}
	if (DebugLevel > 0)
		Print("XML file open must be okay");
	
	// Read in the whole XML file
	// Ave file length == ~7K, so 65536 should always read whole file
	sData = FileReadString(xmlHandle, 65536);	
	FileClose(xmlHandle);

	// Get the currency pair, and split it into the two countries
	string pair = Symbol();
	string cntry1;
	string cntry2;
	if ( 
	     pair == "GOLD" ||
	     StringSubstr(pair, 0, 3)=="OIL" ||
	     StringSubstr(pair, 0, 2)=="CL" ||	  
	     StringSubstr(pair, 0, 3)=="DOW" ||	        
	     StringSubstr(pair, 0, 2)=="DJ" ||	 
	     StringSubstr(pair, 0, 2)=="SP" ||	         
	     StringSubstr(pair, 0, 2)=="ND" ||	  
	     StringSubstr(pair, 0, 2)=="GC"   	     	          	     
	     ) 
	{
	  cntry1 = "ALL";
	  cntry2 = "ALL";	
	}
	else
	{
	  cntry1 = StringSubstr(pair, 0, 3);
	  cntry2 = StringSubstr(pair, 3, 3);
	}  
	if (DebugLevel > 0)
		Print("cntry1 = ", cntry1, "    cntry2 = ", cntry2);
	
	if (DebugLevel > 0)
		Log("Weekly calendar for " + pair + "\n\n");

	// -------------------------------------------------
	// Parse the XML file looking for an event to report
	// -------------------------------------------------
	
	tmpMins = 10080;	// (a week)
	BoEvent = 0;

	string sTags[7] = { "<title>", "<country>", "<date>", "<time>", "<impact>", "<forecast>", "<previous>" };
	string eTags[7] = { "</title>", "</country>", "</date>", "</time>", "</impact>", "</forecast>", "</previous>" };


   ////////////////////////////////////////////////////////////////////////////////////////////////1
	while (true)
	{
		BoEvent = StringFind(sData, "<event>", BoEvent);
		if (BoEvent == -1)
			break;
			
		BoEvent += 7;	
		next = StringFind(sData, "</event>", BoEvent);
		myEvent = StringSubstr(sData, BoEvent, next - BoEvent);
		BoEvent = next;
		
		begin = 0;
		skip = false;
		for (i=0; i < 7; i++)
		{
			mainData[newsIdx][i] = "";
			next = StringFind(myEvent, sTags[i], begin);
			
			// Within this event, if tag not found, then it must be missing; skip it
			if (next == -1)
				continue;
			else
			{
				// We must have found the sTag okay...
				begin = next + StringLen(sTags[i]);			// Advance past the start tag
				end = StringFind(myEvent, eTags[i], begin);	// Find start of end tag
				if (end > begin && end != -1)
				{
					// Get data between start and end tag
					mainData[newsIdx][i] = StringSubstr(myEvent, begin, end - begin);
				}
			}
		}
		
      //		for (i=6; i >= 0; i--)
      //			Print(sTags[i], "  =  ", mainData[newsIdx][i]);

		// = - =   = - =   = - =   = - =   = - =   = - =   = - =   = - =   = - =   = - =
		// Test against filters that define whether we want to 
		// skip this particular annoucemnt
		if(cntry1 !="ALL" && cntry2 !="ALL")
		  if (cntry1 != mainData[newsIdx][COUNTRY] && cntry2 != mainData[newsIdx][COUNTRY])
			 skip = true;
		if (!IncludeHigh && mainData[newsIdx][IMPACT] == "High") 
			skip = true;
		if (!IncludeMedium && mainData[newsIdx][IMPACT] == "Medium") 
			skip = true;
		if (!IncludeLow && mainData[newsIdx][IMPACT] == "Low") 
			skip = true;
		if (!IncludeSpeaks && (StringFind(mainData[newsIdx][TITLE], "speaks") != -1 || 
								StringFind(mainData[newsIdx][TITLE], "Speaks") != -1) ) 
			skip = true;
		if (mainData[newsIdx][TIME] == "All Day" || 
			mainData[newsIdx][TIME] == "Tentative" ||
			mainData[newsIdx][TIME] == "") 
			skip = true;
		// = - =   = - =   = - =   = - =   = - =   = - =   = - =   = - =   = - =   = - =


		// If not skipping this event, then log it into the draw buffers
		if (!skip)
		{
			// If we got this far then we need to calc the minutes until this event
			//
			// First, convert the announcement time to seconds, GMT
			newsTime = StrToTime(MakeDateTime(mainData[newsIdx][DATE], mainData[newsIdx][TIME]));
			
			// Now calculate the minutes until this announcement (may be negative)
			minsTillNews = (newsTime - TimeGMT()) / 60;

			// Keep track of the most recent news announcement.
			// Do that by saving each one until we get to the 
			// first annoucement that isn't in the past; i.e.
			// minsTillNews > 0.  Then, keep this one instead for
			// display, but only once the minutes until the next 
			// news is SMALLER than the minutes since the last.
//			Print("Mins till event: ", minsTillNews);
			if (minsTillNews < 0 || MathAbs(tmpMins) > minsTillNews)
			{
				idxOfNext = newsIdx;
				tmpMins	= minsTillNews;
			}
			
			Log("Weekly calendar for " + pair + "\n\n");
			if (DebugLevel > 0)
			{
				Log("FOREX FACTORY\nTitle: "  + mainData[newsIdx][TITLE] + 
									"\nCountry: "  + mainData[newsIdx][COUNTRY] + 
									"\nDate: "     + mainData[newsIdx][DATE] + 
									"\nTime: "     + mainData[newsIdx][TIME] + 
									"\nImpact: "   + mainData[newsIdx][IMPACT] + 
									"\nForecast: " + mainData[newsIdx][FORECAST] + 
									"\nPrevious: " + mainData[newsIdx][PREVIOUS] + "\n\n");
			}
			
			// Do alert if user has enabled
			string strNextNews=mainData[newsIdx][TITLE];
			datetime NewsTimeServer=newsTime+TimeZoneServer()*3600;	
         if (bRecordTickLog)
         {
           if((minsTillNews>=-5) && (minsTillNews<=5) )
           {
      	    string strFilename = "news\\_"+ Symbol() + " " + strNextNews + " " + TimeToStr(newsTime) + " GMT0.csv";
             int handle = FileOpen(CleanFilename(strFilename), FILE_CSV|FILE_READ|FILE_WRITE, ',' );
             FileSeek(handle,0,SEEK_END);
             FileWrite(handle,MyTimeToString(CurTime()),Bid,Ask,Ask-Bid,(Ask-Bid)/Point,Volume);//,iVolume("GOLD",PERIOD_M1,0));
             FileClose(handle);
           }  
         }				
         if (bRecordScreenShot)
         {  
           if(minsTillNews==0)
           {         
      	    string strFilename2 = "news\\_"+ Symbol() + " " + strNextNews + " " + TimeToStr(newsTime) + " GMT0.gif";
      	    int handle2 = FileOpen(CleanFilename(strFilename2), FILE_BIN|FILE_READ);
      	    if(handle2 == -1)
               WindowScreenShot(CleanFilename(strFilename2),640,480,-1,4,CHART_BAR);
               	
             FileClose(handle2);  
           }    	
           if(minsTillNews==-96)
           {         
      	    string strFilename3 = "news\\_"+ Symbol() + " " + strNextNews + " " + TimeToStr(newsTime) + "_GMT0.gif";
      	    int handle3 = FileOpen(CleanFilename(strFilename3), FILE_BIN|FILE_READ);
      	    if(handle3 == -1)
               WindowScreenShot(CleanFilename(strFilename3),640,480,-1,4,CHART_BAR);
               	
             FileClose(handle2);  
           }    	           
         }      	
					
			if (Alert1MinsBefore != -1 && minsTillNews <= Alert1MinsBefore)
			{
				//Alert(Alert1MinsBefore, " minutes until news for ", pair, ": ", mainData[newsIdx][TITLE]);
				ObjectDelete("_"+strNextNews);
            ObjectCreate("_"+strNextNews, OBJ_TEXT, 0, NewsTimeServer, Close[0]);
            if(ImpactToNumber(mainData[newsIdx][IMPACT])==3)
              ObjectSet("_"+strNextNews, OBJPROP_COLOR, Red);          
            if(ImpactToNumber(mainData[newsIdx][IMPACT])==2)
              ObjectSet("_"+strNextNews, OBJPROP_COLOR, Orange);          
            if(ImpactToNumber(mainData[newsIdx][IMPACT])==1)
              ObjectSet("_"+strNextNews, OBJPROP_COLOR, Yellow);          
            ObjectSetText("_"+strNextNews, strNextNews, 8);          
            ObjectSet("_"+strNextNews, OBJPROP_ANGLE, 90);    

				ObjectDelete(strNextNews);                                                		
     		   ObjectCreate(strNextNews, OBJ_VLINE, 0, NewsTimeServer,Close[0]);
            if(ImpactToNumber(mainData[newsIdx][IMPACT])==3)     		   
     		     ObjectSet(strNextNews, OBJPROP_COLOR, Red);
            if(ImpactToNumber(mainData[newsIdx][IMPACT])==2)     		   
     		     ObjectSet(strNextNews, OBJPROP_COLOR, Orange);
            if(ImpactToNumber(mainData[newsIdx][IMPACT])==1)     		   
     		     ObjectSet(strNextNews, OBJPROP_COLOR, Yellow);
     		   ObjectSet(strNextNews, OBJPROP_STYLE, STYLE_DOT);
            //ObjectSet(strNextNews, OBJPROP_BACK, true);     

			}	
				
			// Buffers are set up as so: 
			// ExtMapBuffer0 contains the time UNTIL each announcement (can be negative)
			// e.g. [0] = -372; [1] = 25; [2] = 450; [3] = 1768 (etc.)
			// ExtMapBuffer1[0] has the mintutes since the last annoucement.
			// ExtMapBuffer1[1] has the mintutes until the next annoucement.
			comment_str1="Server Time Zone("+StrTimeZoneServer()+")\n"+
			             "Local Time Zone("+StrTimeZoneLocal()+")\n"+
			             "GMT Time:"+TimeToStr(TimeGMT(),TIME_DATE|TIME_MINUTES)+"\n"+
			             "Server Time:"+TimeToStr(TimeGMT()+TimeZoneServer()*3600,TIME_DATE|TIME_MINUTES)+"\n"+
			             "Local Time:"+TimeToStr(TimeGMT()+TimeZoneLocal()*3600,TIME_DATE|TIME_MINUTES);
			  
         comment_str2="";//"Server Time Zone("+StrTimeZoneServer()+")\n"+
			             //"Local Time Zone("+StrTimeZoneLocal()+")\n"+
			             //"GMT Time:"+TimeToStr(TimeGMT(),TIME_DATE|TIME_MINUTES);
			   			         
         //  comment_str1=CleanStr("-23.5");

			
			ExtMapBuffer0[newsIdx] = minsTillNews;
			ExtMapBuffer3[newsIdx] = newsIdx;  // newsIdx
         ExtMapBuffer4[newsIdx] = newsTime;  // NewsDateTime,GMT		
         ExtMapBuffer5[newsIdx] = ImpactToNumber(mainData[newsIdx][IMPACT]);  // Impact
         ExtMapBuffer6[newsIdx] = PreviousToNumber(mainData[newsIdx][PREVIOUS]);//
         ExtMapBuffer7[newsIdx] = ForecastToNumber(mainData[newsIdx][FORECAST]);//
			newsIdx++;
		}
	}
   ////////////////////////////////////////////////////////////////////////////////////////////
	// Cycle through the events array and pick out the most recent 
	// past and the next coming event to put into ExtMapBuffer1. 
	// Put the corresponding impact for these two into ExtMapBuffer2.
	bool first = true;
	ExtMapBuffer1[0] = 0;
	ExtMapBuffer1[1] = 0;
	ExtMapBuffer2[0] = 0;
	ExtMapBuffer2[1] = 0;
	string outNews = "Minutes until news events for " + pair + " : ";
	for (i=0; i < newsIdx; i++)	
	{
		outNews = outNews + ExtMapBuffer0[i] + ", ";
		if (ExtMapBuffer0[i] > 0 && first)
		{
			first = false;
			
			// Put the relevant info into the indicator buffers...
			if (i > 0)
			{
				ExtMapBuffer1[0] = ExtMapBuffer0[i-1];
				ExtMapBuffer2[0] = ImpactToNumber(mainData[i-1][IMPACT]);
			}
			ExtMapBuffer1[1] = ExtMapBuffer0[i];
			ExtMapBuffer2[1] = ImpactToNumber(mainData[i][IMPACT]);
		}
		
		// Also use this loop to set which information to display
		if (i == idxOfNext)////////////////////////////////////////
		{
			dispTitle[0]	= mainData[i][TITLE];
			dispCountry[0] 	= mainData[i][COUNTRY];
			dispImpact[0] 	= mainData[i][IMPACT];
			dispForecast[0] = mainData[i][FORECAST];
			dispPrevious[0] = mainData[i][PREVIOUS];
			dispMinutes[0] 	= ExtMapBuffer0[i];
		}
		if (i == idxOfNext + 1)////////////////////////////////////
		{
			dispTitle[1]	= mainData[i][TITLE];
			dispCountry[1] 	= mainData[i][COUNTRY];
			dispImpact[1] 	= mainData[i][IMPACT];
			dispForecast[1] = mainData[i][FORECAST];
			dispPrevious[1] = mainData[i][PREVIOUS];
			dispMinutes[1] 	= ExtMapBuffer0[i];
		}
	}	
	// If we are past all news events, then neither one will have been 
	// set, so set the past event to the last (negative) minutes
	if (ExtMapBuffer1[0] == 0 && ExtMapBuffer1[1] == 0)
		ExtMapBuffer1[0] = ExtMapBuffer0[i-1];

	
	// For debugging...Print the tines until news events, as a "Comment"
	if (DebugLevel > 0)
	{
		Print(outNews);
		Print("LastMins (ExtMapBuffer1[0]) = ", ExtMapBuffer1[0]);
		Print("NextMins (ExtMapBuffer1[1]) = ", ExtMapBuffer1[1]);
	}
	 
    if (TextOutput)
      if(Text)
        OutputToChart_text();
      else
        Comment("");  
    else  
      if(Text)
        OutputToChart();

	if (logHandle != -1)
		FileClose(logHandle);
	return (0);
}


void OutputToChart()
{
	// Added by Robert for using TxtSize and TxtColor for easier reading
	int curY = 10;
	ObjectCreate("Separator1", OBJ_LABEL, 0, 0, 0);
	ObjectSetText("Separator1", dispSeparator1, TxtSize, "Arial Bold", TxtColorTitle);
	ObjectSet("Separator1", OBJPROP_CORNER, 0);
	ObjectSet("Separator1", OBJPROP_XDISTANCE, 10);
	ObjectSet("Separator1", OBJPROP_YDISTANCE, curY); 

	curY = curY + TxtSize + 4;
	ObjectCreate("Sponsor", OBJ_LABEL, 0, 0, 0);
	ObjectSetText("Sponsor", commentStr, TxtSize, "Arial Bold", TxtColorTitle);
	ObjectSet("Sponsor", OBJPROP_CORNER, 0);
	ObjectSet("Sponsor", OBJPROP_XDISTANCE, 10);
	ObjectSet("Sponsor", OBJPROP_YDISTANCE, curY);       

	curY = curY + TxtSize + 4;
	ObjectCreate("Separator2", OBJ_LABEL, 0, 0, 0);
	ObjectSetText("Separator2", dispSeparator1, TxtSize, "Arial Bold", TxtColorTitle);
	ObjectSet("Separator2", OBJPROP_CORNER, 0);
	ObjectSet("Separator2", OBJPROP_XDISTANCE, 10);
	ObjectSet("Separator2", OBJPROP_YDISTANCE, curY);   

	curY = curY + TxtSize + 4;
	ObjectCreate("Minutes", OBJ_LABEL, 0, 0, 0);
	ObjectSetText("Minutes", dispMinutes[0] + " Mins until " + dispCountry[0] + ": " + dispTitle[0], TxtSize, "Arial Bold", TxtColorNews);
	ObjectSet("Minutes", OBJPROP_CORNER, 0);
	ObjectSet("Minutes", OBJPROP_XDISTANCE, 10);
	ObjectSet("Minutes", OBJPROP_YDISTANCE, curY);

	curY = curY + TxtSize + 4;
	ObjectCreate("Impact", OBJ_LABEL, 0, 0, 0);
	ObjectSetText("Impact", "Impact: " + dispImpact[0], TxtSize, "Arial Bold", TxtColorImpact);
	ObjectSet("Impact", OBJPROP_CORNER, 0);
	ObjectSet("Impact", OBJPROP_XDISTANCE, 10);
	ObjectSet("Impact", OBJPROP_YDISTANCE, curY);

	if (dispPrevious[0] != "")
	{
		curY = curY + TxtSize + 4;
		ObjectCreate("Previous", OBJ_LABEL, 0, 0, 0);
		ObjectSetText("Previous", "Previous: " + dispPrevious[0], TxtSize, "Arial Bold", TxtColorPrevious);
		ObjectSet("Previous", OBJPROP_CORNER, 0);
		ObjectSet("Previous", OBJPROP_XDISTANCE, 10);
		ObjectSet("Previous", OBJPROP_YDISTANCE, curY);
	}
	
	if (dispForecast[0] != "")
	{
		curY = curY + TxtSize + 4;
		ObjectCreate("Forecast", OBJ_LABEL, 0, 0, 0);
		ObjectSetText("Forecast", "Forecast: " + dispForecast[0], TxtSize, "Arial Bold", TxtColorForecast);
		ObjectSet("Forecast", OBJPROP_CORNER, 0);
		ObjectSet("Forecast", OBJPROP_XDISTANCE, 10);
		ObjectSet("Forecast", OBJPROP_YDISTANCE, curY); 
	}
	
	// Do second news event
	if (ShowNextTwoEvents)
	{
		curY = curY + TxtSize + 20;
		ObjectCreate("Minutes2", OBJ_LABEL, 0, 0, 0);
		ObjectSetText("Minutes2", dispMinutes[1] + " Mins until " + dispCountry[1] + ": " + dispTitle[1], TxtSize, "Arial Bold", TxtColorNews);
		ObjectSet("Minutes2", OBJPROP_CORNER, 0);
		ObjectSet("Minutes2", OBJPROP_XDISTANCE, 10);
		ObjectSet("Minutes2", OBJPROP_YDISTANCE, curY);

		curY = curY + TxtSize + 4;
		ObjectCreate("Impact2", OBJ_LABEL, 0, 0, 0);
		ObjectSetText("Impact2", "Impact: " + dispImpact[1], TxtSize, "Arial Bold", TxtColorImpact);
		ObjectSet("Impact2", OBJPROP_CORNER, 0);
		ObjectSet("Impact2", OBJPROP_XDISTANCE, 10);
		ObjectSet("Impact2", OBJPROP_YDISTANCE, curY);

		if (dispPrevious[1] != "")
		{
			curY = curY + TxtSize + 4;
			ObjectCreate("Previous2", OBJ_LABEL, 0, 0, 0);
			ObjectSetText("Previous2", "Previous: " + dispPrevious[1], TxtSize, "Arial Bold", TxtColorPrevious);
			ObjectSet("Previous2", OBJPROP_CORNER, 0);
			ObjectSet("Previous2", OBJPROP_XDISTANCE, 10);
			ObjectSet("Previous2", OBJPROP_YDISTANCE, curY);
		}
		
		if (dispForecast[1] != "")
		{
			curY = curY + TxtSize + 4;
			ObjectCreate("Forecast2", OBJ_LABEL, 0, 0, 0);
			ObjectSetText("Forecast2", "Forecast: " + dispForecast[1], TxtSize, "Arial Bold", TxtColorForecast);
			ObjectSet("Forecast2", OBJPROP_CORNER, 0);
			ObjectSet("Forecast2", OBJPROP_XDISTANCE, 10);
			ObjectSet("Forecast2", OBJPROP_YDISTANCE, curY); 
		}
	}
}


void OutputToChart_text()
{

	ObjectDelete("Sponsor"); 
	ObjectDelete("Minutes"); 
	ObjectDelete("Impact");
	ObjectDelete("Previous"); 
	ObjectDelete("Forecast");
	ObjectDelete("Separator1"); 
	ObjectDelete("Separator2");

	ObjectDelete("Sponsor2"); 
	ObjectDelete("Minutes2"); 
	ObjectDelete("Impact2");
	ObjectDelete("Previous2"); 
	ObjectDelete("Forecast2");

   string str1=comment_str1+"\n"+dispSeparator1
               +"\n"+commentStr
               +"\n"+dispSeparator1
               +"\n"+dispMinutes[0] + " Mins until " + dispCountry[0] + ": " + dispTitle[0]
               +"\n"+"Impact: " + dispImpact[0];

	if (dispPrevious[0] != "")
	{
		str1=str1+"\n"+"Previous: " + dispPrevious[0];
   }
	
	if (dispForecast[0] != "")
	{
		str1=str1+"\n"+"Forecast: " + dispForecast[0];
	}

	// Do second news event
	if (ShowNextTwoEvents)
	{
		str1=str1+"\n"+dispMinutes[1] + " Mins until " + dispCountry[1] + ": " + dispTitle[1]

		+"\n"+"Impact: " + dispImpact[1];

	   if (dispPrevious[1] != "")
 	   {
			str1=str1+"\n"+"Previous: " + dispPrevious[1];
		}
		
		if (dispForecast[1] != "")
		{
			str1=str1+"\n"+"Forecast: " + dispForecast[1];
		}
	}
	str1=str1+"\n"+dispSeparator1+"\n"+comment_str2;
	//Comment("");
	Comment(str1);
}



int ImpactToNumber(string impact)
{
	if (impact == "High")
		return (3);
	if (impact == "Medium")
		return (2);
	if (impact == "Low")
		return (1);
	else
		return (0);
}

double PreviousToNumber(string Previous)
{
  return(StrToDouble(CleanStr(Previous)));  // Previous);  // Forecast);
}

double ForecastToNumber(string Forecast)
{
  return(StrToDouble(CleanStr(Forecast)));  // Forecast);
}
/*
ascii[56, 57, 58, 59] = '89:;'
ascii[52, 53, 54, 55] = '4567'
ascii[48, 49, 50, 51] = '0123'
ascii[44, 45, 46, 47] = ',-./'
*/
string CleanStr(string str)
{
	// change illegal filename characters in str to '-'
	int i;
   string r1;
   
   for (i=0;i<StringLen(str);i++)
   {
     if((58>StringGetChar(str,i)) && (47<StringGetChar(str,i)))
       r1=r1+CharToStr(StringGetChar(str,i));
     if((CharToStr(StringGetChar(str,i))=="."))  
       r1=r1+CharToStr(StringGetChar(str,i));
     if((CharToStr(StringGetChar(str,i))=="-"))  	 
       r1=r1+CharToStr(StringGetChar(str,i));
   }
	return(r1);
}

   
string MakeDateTime(string strDate, string strTime)
{
	// converts forexfactory time & date into yyyy.mm.dd hh:mm
	int n1stDash = StringFind(strDate, "-");
	int n2ndDash = StringFind(strDate, "-", n1stDash+1);

	string strMonth = StringSubstr(strDate, 0, 2);
	string strDay = StringSubstr(strDate, 3, 2);
	string strYear = StringSubstr(strDate, 6, 2); 
	strYear = "20" + strYear;
	
	int nTimeColonPos	= StringFind(strTime, ":");
	string strHour = StringSubstr(strTime, 0, nTimeColonPos);
	string strMinute = StringSubstr(strTime, nTimeColonPos+1, 2);
	string strAM_PM = StringSubstr(strTime, StringLen(strTime)-2);

	int nHour24 = StrToInteger(strHour);
	if (strAM_PM == "pm" || strAM_PM == "PM" && nHour24 != 12)
	{
		nHour24 += 12;
	}
	string strHourPad = "";
	if (nHour24 < 10) 
		strHourPad = "0";

	return(StringConcatenate(strYear, ".", strMonth, ".", strDay, " ", strHourPad, nHour24, ":", strMinute));
}


//=================================================================================================
//=================================================================================================
//====================================   GrabWeb Functions   ======================================
//=================================================================================================
//=================================================================================================
// Main Webscraping function
// ~~~~~~~~~~~~~~~~~~~~~~~~~
// bool GrabWeb(string strUrl, string& strWebPage)
// returns the text of any webpage. Returns false on timeout or other error
// 
// Parsing functions
// ~~~~~~~~~~~~~~~~~
// string GetData(string strWebPage, int nStart, string strLeftTag, string strRightTag, int& nPos)
// obtains the text between two tags found after nStart, and sets nPos to the end of the second tag
//
// void Goto(string strWebPage, int nStart, string strTag, int& nPos)
// Sets nPos to the end of the first tag found after nStart 

bool bWinInetDebug = false;

int hSession_IEType;
int hSession_Direct;
int Internet_Open_Type_Preconfig = 0;
int Internet_Open_Type_Direct = 1;
int Internet_Open_Type_Proxy = 3;
int Buffer_LEN = 13;

#import "wininet.dll"

#define INTERNET_FLAG_PRAGMA_NOCACHE    0x00000100 // Forces the request to be resolved by the origin server, even if a cached copy exists on the proxy.
#define INTERNET_FLAG_NO_CACHE_WRITE    0x04000000 // Does not add the returned entity to the cache. 
#define INTERNET_FLAG_RELOAD            0x80000000 // Forces a download of the requested file, object, or directory listing from the origin server, not from the cache.

int InternetOpenA(
	string 	sAgent,
	int		lAccessType,
	string 	sProxyName="",
	string 	sProxyBypass="",
	int 	lFlags=0
);

int InternetOpenUrlA(
	int 	hInternetSession,
	string 	sUrl, 
	string 	sHeaders="",
	int 	lHeadersLength=0,
	int 	lFlags=0,
	int 	lContext=0 
);

int InternetReadFile(
	int 	hFile,
	string 	sBuffer,
	int 	lNumBytesToRead,
	int& 	lNumberOfBytesRead[]
);

int InternetCloseHandle(
	int 	hInet
);
#import


int hSession(bool Direct)
{
	string InternetAgent;
	if (hSession_IEType == 0)
	{
		InternetAgent = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Q312461)";
		hSession_IEType = InternetOpenA(InternetAgent, Internet_Open_Type_Preconfig, "0", "0", 0);
		hSession_Direct = InternetOpenA(InternetAgent, Internet_Open_Type_Direct, "0", "0", 0);
	}
	if (Direct) 
	{ 
		return(hSession_Direct); 
	}
	else 
	{
		return(hSession_IEType); 
	}
}


bool GrabWeb(string strUrl, string& strWebPage)
{
	int 	hInternet;
	int		iResult;
	int 	lReturn[]={1};
	string 	sBuffer="x";
	int 	bytes;
	
	hInternet = InternetOpenUrlA(hSession(FALSE), strUrl, "0", 0, 
								INTERNET_FLAG_NO_CACHE_WRITE | 
								INTERNET_FLAG_PRAGMA_NOCACHE | 
								INTERNET_FLAG_RELOAD, 0);
								
	if (bWinInetDebug) 
		Log("hInternet: " + hInternet);   
	if (hInternet == 0) 
		return(false);
		
	iResult = InternetReadFile(hInternet, sBuffer, Buffer_LEN, lReturn);
	
	if (bWinInetDebug) 
		Log("iResult: " + iResult);
	if (bWinInetDebug) 
		Log("lReturn: " + lReturn[0]);
	if (bWinInetDebug) 
		Log("iResult: " + iResult);
	if (bWinInetDebug) 
		Log("sBuffer: " +  sBuffer);
	if (iResult == 0) 
		return(false);
	bytes = lReturn[0];

	strWebPage = StringSubstr(sBuffer, 0, lReturn[0]);
	
	// If there's more data then keep reading it into the buffer
	while (lReturn[0] != 0)
	{
		iResult = InternetReadFile(hInternet, sBuffer, Buffer_LEN, lReturn);
		if (lReturn[0]==0) 
			break;
		bytes = bytes + lReturn[0];
		strWebPage = strWebPage + StringSubstr(sBuffer, 0, lReturn[0]);
	}

	iResult = InternetCloseHandle(hInternet);
	if (iResult == 0) 
		return(false);
	if (bWinInetDebug) 
	{ 	
		Log("iResult: " + iResult);
		int handle = FileOpen("Grabweb.htm",FILE_BIN|FILE_READ|FILE_WRITE);
		FileWriteString(handle,strWebPage,StringLen(strWebPage));
		FileClose(handle);
	}
	return(true);
}


/*
string GetData(string strWebPage, int nStart, string strLeftTag, string strRightTag, int& nPos)
{
	nPos = StringFind(strWebPage,strLeftTag,nStart);
	if (nPos == -1)
	{
		if (bWinInetDebug) Log("GetData(WebPage,"+nStart+","+strLeftTag+","+strRightTag+"): not found, returning empty string and -1");
		return("");
	}
	nPos += StringLen(strLeftTag);
	int nEnd = StringFind(strWebPage,strRightTag,nPos+1);
	string strData = StringTrimLeft(StringTrimRight(StringSubstr(strWebPage,nPos,nEnd-nPos)));
	if (bWinInetDebug) Log("GetData(WebPage,"+nStart+","+strLeftTag+","+strRightTag+") returning "+strData+" and "+nPos);
	return(strData);
}

void Goto(string strWebPage, int nStart, string strTag, int& nPos)
{
	nPos = StringFind(strWebPage,strTag,nStart);
	if (nPos == -1) 
	{
		if (bWinInetDebug) Log("Goto("+nStart+","+strTag+") not found. Returning -1");
		return;
	}
	nPos += StringLen(strTag);
	if (bWinInetDebug) Log("Goto("+nStart+","+strTag+") returning "+nPos);
}

void SaveWebPage(string strWebPage, string strID)
{
	string strFilename = strID+"_"+Year()+Pad(Month())+Pad(Day())+"-"+Pad(Hour())+Pad(Minute())+Pad(Seconds())+".htm";
	int handle = FileOpen(strFilename,FILE_BIN|FILE_WRITE);
	FileWriteString(handle,strWebPage,StringLen(strWebPage));
   FileClose(handle);
}


string Pad(int n)
{
	string strPad = "";
	if (n <= 9) strPad = "0";
	return(strPad+n);
}
*/

//=================================================================================================
//=================================================================================================
//===================================   LogUtils Functions   ======================================
//=================================================================================================
//=================================================================================================

void OpenLog(string strName)
{
	if (!EnableLogging) 
		return;

	if (logHandle <= 0)
	{
		string strMonthPad = "";
 	 	string strDayPad = "";
  		if (Month() < 10) 
  			strMonthPad = "0";
  		if (Day() < 10) 
  			strDayPad = "0";
  			
  		string strFilename = StringConcatenate(strName, "_", Year(), strMonthPad, Month(), strDayPad, Day(), "_log.txt");
  		
		logHandle = FileOpen(strFilename,FILE_CSV|FILE_READ|FILE_WRITE);
		Print("logHandle =================================== ", logHandle);
	}
	if (logHandle > 0)
	{
		FileFlush(logHandle);
		FileSeek(logHandle, 0, SEEK_END);
	}
}


void Log(string msg)
{
	if (!EnableLogging) 
		return;
		
	if (logHandle <= 0) 
		return;
		
	msg = TimeToStr(TimeCurrent(),TIME_DATE|TIME_MINUTES|TIME_SECONDS) + " " + msg;
	FileWrite(logHandle,msg);
}

//=================================================================================================
//=================================================================================================
//===================================   Timezone Functions   ======================================
//=================================================================================================
//=================================================================================================


#import "kernel32.dll"
int  GetTimeZoneInformation(int& TZInfoArray[]);
#import

#define TIME_ZONE_ID_UNKNOWN   0
#define TIME_ZONE_ID_STANDARD  1
#define TIME_ZONE_ID_DAYLIGHT  2

// Local timezone in hours, adjusting for daylight saving
double TimeZoneLocal()
{
	int TZInfoArray[43];

	switch(GetTimeZoneInformation(TZInfoArray))
	{
	case TIME_ZONE_ID_UNKNOWN: 
		Print("Error obtaining PC timezone from GetTimeZoneInformation in kernel32.dll. Returning 0");
		return(0);

	case TIME_ZONE_ID_STANDARD:
		return(TZInfoArray[0]/(-60.0));
	
	case TIME_ZONE_ID_DAYLIGHT:
		return((TZInfoArray[0]+TZInfoArray[42])/(-60.0));
		
	default:
		Print("Unkown return value from GetTimeZoneInformation in kernel32.dll. Returning 0");
		return(0);
	}
}

string StrTimeZoneLocal()
{
  string r1;
  if(TimeZoneLocal()>=0)
    r1="GMT+"+DoubleToStr(TimeZoneLocal(),0);
  else
    r1="GMT"+DoubleToStr(TimeZoneLocal(),0);    
  return(r1);   
}



// Server timezone in hours
double TimeZoneServer()
{
	int ServerToLocalDiffMinutes = (TimeCurrent()-TimeLocal())/60;
	
	// round to nearest 30 minutes to allow for inaccurate PC clock
	int nHalfHourDiff = MathRound(ServerToLocalDiffMinutes/30.0);
	ServerToLocalDiffMinutes = nHalfHourDiff*30;
	return(TimeZoneLocal() + ServerToLocalDiffMinutes/60.0);
}

string StrTimeZoneServer()
{
  string r1;
  if(TimeZoneServer()>=0)
    r1="GMT+"+DoubleToStr(TimeZoneServer(),0);
  else
    r1="GMT"+DoubleToStr(TimeZoneServer(),0);    
  return(r1);   
}

// Uses local PC time, local PC timezone, and server time to calculate GMT time at arrival of last tick
datetime TimeGMT()
{
	// two ways of calculating
	// 1. From PC time, which may not be accurate
	// 2. From server time. Most accurate except when server is down on weekend
	datetime dtGmtFromLocal = TimeLocal() - TimeZoneLocal()*3600;
	datetime dtGmtFromServer = TimeCurrent() - TimeZoneServer()*3600;

	// return local-derived value if server value is out by more than 5 minutes, eg during weekend
	if (dtGmtFromLocal > dtGmtFromServer + 300)
	{
		return(dtGmtFromLocal);
	}
	else
	{
		return(dtGmtFromServer);
	}	
}

//=================================================================================================
//=================================================================================================
//=================================   END IMPORTED FUNCTIONS  =====================================
//=================================================================================================
//=================================================================================================

void create_globals()
{

   string Global1 = /*Symbol() + */"_GMTdiffServer";
   if (! GlobalVariableCheck(Global1))
     GlobalVariableSet(Global1,TimeZoneServer()); //Global Symbol Comment
   string Global2 = /*Symbol() +*/ "_GMTdiffLocal";
   if (! GlobalVariableCheck(Global2))
     GlobalVariableSet(Global2,TimeZoneLocal()); //Global Symbol Comment
}


void deinit_globals()
{

   string Global1 = /*Symbol() + */"_GMTdiffServer";
   if (GlobalVariableCheck(Global1))
     GlobalVariableDel(Global1); //Global Symbol Comment
   string Global2 = /*Symbol() +*/ "_GMTdiffLocal";
   if (GlobalVariableCheck(Global2))
     GlobalVariableDel(Global2); //Global Symbol Comment
    
}

//////////////////////////////////////////////////////

void init_global1() //100,7
{
         string Global1 = /*Symbol() + */"_GMTdiffServer";
         if (GlobalVariableCheck(Global1))
           GlobalVariableSet(Global1,TimeZoneServer());

}

void init_global2() //100,7
{
         string Global2 = /*Symbol() +*/ "_GMTdiffLocal";
         if (GlobalVariableCheck(Global2))
           GlobalVariableSet(Global2,TimeZoneLocal());

}

double get_global1()
{
        double r1;
        string Global1 = /*Symbol() + */"_GMTdiffServer";
        if (GlobalVariableCheck(Global1))
          r1=GlobalVariableGet(Global1);
        else
          r1=0;  
        return(r1);
}

double get_global2()
{
         double r2;
         string Global2 = /*Symbol() +*/ "_GMTdiffLocal";
         if (GlobalVariableCheck(Global2))
           r2=GlobalVariableGet(Global2);
         else
           r2=0;  
         return(r2);
}


/////////////////////////////////////////////////////
//
/////////////////////////////////////////////////////
string CleanFilename(string str)
{
	// change illegal filename characters in str to '-'
	int nPos; 

	string strRet = str;
	nPos = StringFind(str,"/");
	if (nPos != -1) strRet = StringSetChar(strRet, nPos, '-');
	
	//nPos = StringFind(strRet,"\\");
	//if (nPos != -1) strRet = StringSetChar(strRet, nPos, '-');
	
	nPos = StringFind(strRet,":");
	if (nPos != -1) strRet = StringSetChar(strRet, nPos, '-');

	nPos = StringFind(strRet,"*");
	if (nPos != -1) strRet = StringSetChar(strRet, nPos, '-');

	nPos = StringFind(strRet,"?");
	if (nPos != -1) strRet = StringSetChar(strRet, nPos, ' ');

	nPos = StringFind(strRet,"\"");
	if (nPos != -1) strRet = StringSetChar(strRet, nPos, '-');

	nPos = StringFind(strRet,">");
	if (nPos != -1) strRet = StringSetChar(strRet, nPos, '-');

	nPos = StringFind(strRet,"<");
	if (nPos != -1) strRet = StringSetChar(strRet, nPos, '-');

	nPos = StringFind(strRet,"|");
	if (nPos != -1) strRet= StringSetChar(strRet, nPos, '-');

	return(strRet);
}
string MyTimeToString(datetime dt)
{
   // output format dd/mm/yyyy mm:hh, eg for reading by Excel
   string strDayPad;
   int nDay = TimeDay(dt);
   if (nDay >= 0 && nDay <= 9) strDayPad = "0";
   else strDayPad = "";

   string strMonthPad;
   int nMonth = TimeMonth(dt);
   if (nMonth >= 0 && nMonth <= 9) strMonthPad = "0";
   else strMonthPad = "";
   
   string strHourPad;
   int nHour = TimeHour(dt);
   if (nHour >= 0 && nHour <= 9) strHourPad = "0";
   else strHourPad = "";
   
   string strMinutePad;
   int nMinute = TimeMinute(dt);
   if (nMinute >= 0 && nMinute <= 9) strMinutePad = "0";
   else strMinutePad = "";

   string strSecondPad;
   int nSecond = TimeSeconds(dt);
   if (nSecond >= 0 && nSecond <= 9) strSecondPad = "0";
   else strSecondPad = "";

   return(StringConcatenate(strDayPad,nDay,"/",strMonthPad,nMonth,"/",TimeYear(dt)," ",strHourPad,nHour,":",strMinutePad,nMinute,":",strSecondPad,nSecond));
}

////////////////////////////////////////////////////////////

double news_FF_code(string Curr1,string News1)
{
  double c1,n1,r1;
  
  c1=curr_FF_code(Curr1);
  n1=news1_FF_code(News1);
  r1=c1+n1;
  
  return(r1);

}

int curr_FF_code(string Curr1)
{
  int r1=0;//ALL

if (Curr1=="EUR")
  r1=1000;
else if (Curr1=="USD") 
  r1=2000;
else if (Curr1=="JPY") 
  r1=3000;
else if (Curr1=="GBP") 
  r1=4000;
else if (Curr1=="AUD") 
  r1=5000;
else if (Curr1=="CAD") 
  r1=6000;
else if (Curr1=="CHF") 
  r1=7000;
else if (Curr1=="NZD") 
  r1=8000;
  
  return (r1);
}

int news1_FF_code(string News1)
{
  int r1=0;
if (News1=="ADP Nonfarm Employment Change") r1=1;
else if (News1=="All Industries Activity Index m/m") r1=2;
else if (News1=="All Industries Index m/m") r1=3;
else if (News1=="Atlanta Fed President Guynn Speaks") r1=4;
else if (News1=="Average Cash Earnings y/y") r1=5;
else if (News1=="Average Earnings Index +Bonus q/y") r1=6;
else if (News1=="Average Hourly Earnings m/m") r1=7;
else if (News1=="Average Weekly Earnings q/q") r1=8;
else if (News1=="BBC Quarterly Economic Survey") r1=9;
else if (News1=="BIS Meeting") r1=10;
else if (News1=="BOC Deputy Governor Kennedy Speaks") r1=11;
else if (News1=="BOC Governor Dodge Speaks") r1=12;
else if (News1=="BOC Governor Jenkins Speaks") r1=13;
else if (News1=="BOC Governor Macklem Speaks") r1=14;
else if (News1=="BOC Interest Rate Statement") r1=15;
else if (News1=="BOE Interest Rate Decision") r1=16;
else if (News1=="BOE Interest Rate Statement") r1=17;
else if (News1=="BOE Meeting Minutes") r1=18;
else if (News1=="BOE Quarterly Inflation Report") r1=19;
else if (News1=="BOJ Deputy Governor Muto Speaks") r1=20;
else if (News1=="BOJ Governor Fukui Speaks") r1=21;
else if (News1=="BOJ Interest Rate Decision") r1=22;
else if (News1=="BOJ Interest Rate Statement") r1=23;
else if (News1=="BOJ Meeting Minutes") r1=24;
else if (News1=="BOJ Monetary Policy Report") r1=25;
else if (News1=="BRC Retail Sales y/y") r1=26;
else if (News1=="Beige Book") r1=27;
else if (News1=="Bernanke Speaks") r1=28;
else if (News1=="Bernanke Testifies") r1=29;
else if (News1=="Bernanke Testifies to Senate") r1=30;
else if (News1=="Boston Fed President Minehan Speaks") r1=31;
else if (News1=="Broad Liquidity y/y") r1=32;
else if (News1=="Building Approvals m/m") r1=33;
else if (News1=="Building Consents") r1=34;
else if (News1=="Building Permits") r1=35;
else if (News1=="Building Permits m/m") r1=36;
else if (News1=="Business Climate Indicator") r1=37;
else if (News1=="Business Conditions Index") r1=38;
else if (News1=="Business Confidence") r1=39;
else if (News1=="Business Inventories") r1=40;
else if (News1=="Business Inventories m/m") r1=41;
else if (News1=="Business Investment q/q (f)") r1=42;
else if (News1=="CBI Distributive Trades Survey") r1=43;
else if (News1=="CBI Industrial Trends Indicator") r1=44;
else if (News1=="CBI Industrial Trends Orders") r1=45;
else if (News1=="CGPI m/m") r1=46;
else if (News1=="CIPS Construction PMI") r1=47;
else if (News1=="CPI excl. Food and Energy m/m") r1=48;
else if (News1=="CPI excl. Fresh Food m/m") r1=49;
else if (News1=="CPI excl. Volatile Items m/m") r1=50;
else if (News1=="CPI m/m") r1=51;
else if (News1=="CPI m/m (f)") r1=52;
else if (News1=="CPI m/m (p)") r1=53;
else if (News1=="CPI q/q") r1=54;
else if (News1=="CPI y/y") r1=55;
else if (News1=="CPI y/y (p)") r1=56;
else if (News1=="Capacity Utilization Rate") r1=57;
else if (News1=="Central Bank Chiefs Speak") r1=58;
else if (News1=="Chicago Fed Pres. Moskow Speaks") r1=59;
else if (News1=="Chicago Fed President Moskow Speaks") r1=60;
else if (News1=="Chicago PMI") r1=61;
else if (News1=="Claimant Count Change") r1=62;
else if (News1=="Cleveland Fed President Pianalto") r1=63;
else if (News1=="Cleveland Fed President Pianalto Speaks") r1=64;
else if (News1=="Coincident Index") r1=65;
else if (News1=="Construction PMI") r1=66;
else if (News1=="Construction Spending m/m") r1=67;
else if (News1=="Construction Work Done q/q") r1=68;
else if (News1=="Consumer Confidence") r1=69;
else if (News1=="Consumer Confidence Index") r1=70;
else if (News1=="Consumer Credit") r1=71;
else if (News1=="Consumer Credit m/m") r1=72;
else if (News1=="Consumer Price Index m/m") r1=73;
else if (News1=="Consumer Price Index m/m (f)") r1=74;
else if (News1=="Consumer Price Index q/q") r1=75;
else if (News1=="Consumer Price Index y/y (p)") r1=76;
else if (News1=="Consumer Sentiment") r1=77;
else if (News1=="Consumer Sentiment (f)") r1=78;
else if (News1=="Consumer Sentiment (p)") r1=79;
else if (News1=="Consumption Indicator") r1=80;
else if (News1=="Core CPI m/m") r1=81;
else if (News1=="Core CPI y/y") r1=82;
else if (News1=="Core Durable Goods Orders m/m") r1=83;
else if (News1=="Core Machinery Orders m/m") r1=84;
else if (News1=="Core PCE Price Index m/m") r1=85;
else if (News1=="Core PPI m/m") r1=86;
else if (News1=="Core Retail Sales m/m") r1=87;
else if (News1=="Core Tokyo CPI m/m") r1=88;
else if (News1=="Corporate Goods Price Index m/m") r1=89;
else if (News1=="Corporate Services Price Index m/m") r1=90;
else if (News1=="Crude Oil Inventories") r1=91;
else if (News1=="Current Account") r1=92;
else if (News1=="Current Account 2Q") r1=93;
else if (News1=="Dallas Fed Pres. Fisher Speaks") r1=94;
else if (News1=="Dallas Fed President Fisher Speaks") r1=95;
else if (News1=="Domestic CGPI m/m") r1=96;
else if (News1=="Domestic Capital Goods Price Index m/m") r1=97;
else if (News1=="Domestic Vehicle Sales") r1=98;
else if (News1=="Durable Goods Orders") r1=99;
else if (News1=="Durable Goods Orders m/m") r1=100;
else if (News1=="Dwelling Commencements q/q") r1=101;
else if (News1=="ECB Central Banking Conference Begins") r1=102;
else if (News1=="ECB Council Member Mersch Speaks") r1=103;
else if (News1=="ECB Gonzlez-Pramo Speaks") r1=104;
else if (News1=="ECB Governor Mersch Speaks") r1=105;
else if (News1=="ECB Interest Rate Decision") r1=106;
else if (News1=="ECB Interest Rate Statement") r1=107;
else if (News1=="ECB Member Tumpel-Gugerell Speaks") r1=108;
else if (News1=="ECB President Trichet Speaks") r1=109;
else if (News1=="EIA Crude Oil Inventories") r1=110;
else if (News1=="EU Current Account") r1=111;
else if (News1=="EU Industrial New Orders m/m") r1=112;
else if (News1=="EU Trade Balance") r1=113;
else if (News1=="EU ZEW Economic Sentiment") r1=114;
else if (News1=="Eco Watchers Survey") r1=115;
else if (News1=="Empire Manufacturing Index") r1=116;
else if (News1=="Empire State Business Conditions Index (") r1=117;
else if (News1=="Employment") r1=118;
else if (News1=="Employment Change") r1=119;
else if (News1=="Employment Change q/q") r1=120;
else if (News1=="Employment Cost Index q/q") r1=121;
else if (News1=="Existing Home Sales") r1=122;
else if (News1=="Existing Home Sales m/m") r1=123;
else if (News1=="Export Prices m/m") r1=124;
else if (News1=="FOMC Interest Rate Statement") r1=125;
else if (News1=="FOMC Meeting Begins") r1=126;
else if (News1=="FOMC Meeting Minutes") r1=127;
else if (News1=="FPI m/m") r1=128;
else if (News1=="Factory Orders m/m") r1=129;
else if (News1=="Factory Orders m/m (p)") r1=130;
else if (News1=="Fed Chairman Bernanke Speaks") r1=131;
else if (News1=="Fed Governor Bies Speaks") r1=132;
else if (News1=="Fed Governor Kohn Speaks") r1=133;
else if (News1=="Fed Governor Kroszner Speaks") r1=134;
else if (News1=="Fed Governor Mishkin Speaks") r1=135;
else if (News1=="Fed Governor Olson Speaks") r1=136;
else if (News1=="Fed Governor Shmidt Speaks") r1=137;
else if (News1=="Fed Governor Warsh Speaks") r1=138;
else if (News1=="Fed Vice Chairman Ferguson Speaks") r1=139;
else if (News1=="Food Price Index m/m") r1=140;
else if (News1=="French Consumer Spending m/m") r1=141;
else if (News1=="French Nonfarm Employment Change q/q") r1=142;
else if (News1=="French Unemployment Rate") r1=143;
else if (News1=="G-20 Meeting Begins") r1=144;
else if (News1=="GDP Annualized q/q") r1=145;
else if (News1=="GDP Annualized q/q (f)") r1=146;
else if (News1=="GDP Deflator Annualized") r1=147;
else if (News1=="GDP Deflator Annualized q/q") r1=148;
else if (News1=="GDP Deflator Annualized q/q (f)") r1=149;
else if (News1=="GDP Deflator q/q") r1=150;
else if (News1=="GDP Deflator q/q (f)") r1=151;
else if (News1=="GDP Deflator q/q (p)") r1=152;
else if (News1=="GDP Deflator q/y") r1=153;
else if (News1=="GDP Deflator q/y (f)") r1=154;
else if (News1=="GDP Deflator q/y (p)") r1=155;
else if (News1=="GDP Deflator y/y (f)") r1=156;
else if (News1=="GDP Forecasts") r1=157;
else if (News1=="GDP m/m") r1=158;
else if (News1=="GDP q/q") r1=159;
else if (News1=="GDP q/q (f)") r1=160;
else if (News1=="GDP q/q (p)") r1=161;
else if (News1=="German CPI m/m (f)") r1=162;
else if (News1=="German CPI m/m (p)") r1=163;
else if (News1=="German Consumer Confidence") r1=164;
else if (News1=="German Consumer Price Index m/m (f)") r1=165;
else if (News1=="German Consumer Price Index m/m (p)") r1=166;
else if (News1=="German Current Account") r1=167;
else if (News1=="German Factory Orders m/m") r1=168;
else if (News1=="German GDP q/q") r1=169;
else if (News1=="German GDP q/q (f)") r1=170;
else if (News1=="German Ifo Business Climate Index") r1=171;
else if (News1=="German Ifo Business Expectations Index") r1=172;
else if (News1=="German Import Price Index m/m") r1=173;
else if (News1=="German Import Prices m/m") r1=174;
else if (News1=="German Industrial Production m/m") r1=175;
else if (News1=="German Manufacturing PMI") r1=176;
else if (News1=="German PPI m/m") r1=177;
else if (News1=="German Producer Price Index m/m") r1=178;
else if (News1=="German Retail PMI") r1=179;
else if (News1=="German Retail Sales m/m") r1=180;
else if (News1=="German Services PMI") r1=181;
else if (News1=="German Trade Balance") r1=182;
else if (News1=="German Unemployment Rate") r1=183;
else if (News1=="German Wholesale Price Index m/m") r1=184;
else if (News1=="German ZEW Economic Sentiment") r1=185;
else if (News1=="GfK Consumer Confidence") r1=186;
else if (News1=="Greenspan Speaks") r1=187;
else if (News1=="Greenspan Testifies") r1=188;
else if (News1=="Gross Domestic Product 2Q") r1=189;
else if (News1=="Gross Domestic Product Annualized (p)") r1=190;
else if (News1=="Gross Domestic Product Forecast") r1=191;
else if (News1=="Gross Domestic Product m/m") r1=192;
else if (News1=="Gross Domestic Product q/q") r1=193;
else if (News1=="Gross Domestic Product q/q (f)") r1=194;
else if (News1=="Gross Domestic Product q/q (p)") r1=195;
else if (News1=="Gross Domestic Product q/y (f)") r1=196;
else if (News1=="Head of International Debelle Speaks") r1=197;
else if (News1=="Holiday: Between Day") r1=198;
else if (News1=="Holiday: Boxing Day") r1=199;
else if (News1=="Holiday: Children's Day") r1=200;
else if (News1=="Holiday: Christmas Day") r1=201;
else if (News1=="Holiday: Columbus Day") r1=202;
else if (News1=="Holiday: Culture Day") r1=203;
else if (News1=="Holiday: Independence Day") r1=204;
else if (News1=="Holiday: Labor Day") r1=205;
else if (News1=="Holiday: Labour Thanksgiving Day") r1=206;
else if (News1=="Holiday: Martin Luther King Day") r1=207;
else if (News1=="Holiday: May Day") r1=208;
else if (News1=="Holiday: Memorial Day") r1=209;
else if (News1=="Holiday: New Year's Day Observed") r1=210;
else if (News1=="Holiday: New Years Day") r1=211;
else if (News1=="Holiday: New Years Holiday") r1=212;
else if (News1=="Holiday: President's Day") r1=213;
else if (News1=="Holiday: Respect for the Aged Day") r1=214;
else if (News1=="Holiday: Saint Stephen's Day") r1=215;
else if (News1=="Holiday: Sports Day") r1=216;
else if (News1=="Holiday: Spring Bank Holiday") r1=217;
else if (News1=="Holiday: Thanksgiving") r1=218;
else if (News1=="Holiday: Thanksgiving Day") r1=219;
else if (News1=="House Price Index") r1=220;
else if (News1=="House Price Index m/m") r1=221;
else if (News1=="House Price Index q/q") r1=222;
else if (News1=="Household Spending y/y") r1=223;
else if (News1=="Housing Starts") r1=224;
else if (News1=="Housing Starts y/y") r1=225;
else if (News1=="ILO Unemployment Rate") r1=226;
else if (News1=="IPPI m/m") r1=227;
else if (News1=="ISM Manufacturing Index") r1=228;
else if (News1=="ISM Manufacturing Prices") r1=229;
else if (News1=="ISM Non-Manufacturing Activity") r1=230;
else if (News1=="ISM Non-Manufacturing Index") r1=231;
else if (News1=="ISM Non-Manufacturing Prices") r1=232;
else if (News1=="Ifo Business Climate Index") r1=233;
else if (News1=="Ifo Business Expectations Index") r1=234;
else if (News1=="Import Price Index m/m") r1=235;
else if (News1=="Import Prices m/m") r1=236;
else if (News1=="Import Prices q/q") r1=237;
else if (News1=="Index of Commodity Prices") r1=238;
else if (News1=="Index of Services 3m/3m") r1=239;
else if (News1=="Industrial Capacity Utilization Rate") r1=240;
else if (News1=="Industrial Confidence") r1=241;
else if (News1=="Industrial New Orders m/m") r1=242;
else if (News1=="Industrial Product Price Index m/m") r1=243;
else if (News1=="Industrial Production m/m") r1=244;
else if (News1=="Industrial Production m/m (f)") r1=245;
else if (News1=="Industrial Production m/m (p)") r1=246;
else if (News1=="Industrial Production q/q") r1=247;
else if (News1=="Interest Rate Statement") r1=248;
else if (News1=="International Securities Transactions") r1=249;
else if (News1=="Iraq Study Group Report") r1=250;
else if (News1=="Ivey PMI") r1=251;
else if (News1=="Kansas City Fed Hoeing Speaks") r1=252;
else if (News1=="Kansas City Fed Pres Hoenig Speaks") r1=253;
else if (News1=="Kansas City Fed Pres. Hoeing Speaks") r1=254;
else if (News1=="Kansas City Fed Pres. Hoenig Speaks") r1=255;
else if (News1=="LOW") r1=256;
else if (News1=="Labor Cost Index q/q") r1=257;
else if (News1=="Labor Productivity q/q") r1=258;
else if (News1=="Large Retailers' Sales y/y") r1=259;
else if (News1=="Leading Economic Index") r1=260;
else if (News1=="Leading Index m/m") r1=261;
else if (News1=="Leading Indicators") r1=262;
else if (News1=="Leading Indicators Index") r1=263;
else if (News1=="Leading Indicators m/m") r1=264;
else if (News1=="M2+CD Money Supply y/y") r1=265;
else if (News1=="M3 Money Supply y/y") r1=266;
else if (News1=="M4 Money Supply m/m") r1=267;
else if (News1=="MED") r1=268;
else if (News1=="Machine Orders m/m") r1=269;
else if (News1=="Machine Tool Orders y/y") r1=270;
else if (News1=="Machinery Tool Orders y/y") r1=271;
else if (News1=="Machinery Tool Orders y/y (f)") r1=272;
else if (News1=="Machinery Tool Orders y/y (p)") r1=273;
else if (News1=="Machinery orders") r1=274;
else if (News1=="Manufacturing PMI") r1=275;
else if (News1=="Manufacturing Production m/m") r1=276;
else if (News1=="Manufacturing Sales q/q") r1=277;
else if (News1=="Manufacturing Shipments") r1=278;
else if (News1=="Manufacturing Shipments m/m") r1=279;
else if (News1=="Merchandise Terms of Trade q/q") r1=280;
else if (News1=="Midterm Elections") r1=281;
else if (News1=="Minneapolis Fed President Stern Speaks") r1=282;
else if (News1=="Mishkin Testifies to Senate") r1=283;
else if (News1=="Monetary Base y/y") r1=284;
else if (News1=="Mortgage Approvals") r1=285;
else if (News1=="Mortgage Equity Withdrawal") r1=286;
else if (News1=="NAB Business Confidence") r1=287;
else if (News1=="NAHB Housing Market Index") r1=288;
else if (News1=="NY Fed President Geithner Speaks") r1=289;
else if (News1=="National Day of Mourning") r1=290;
else if (News1=="Nationwide Consumer Confidence Index") r1=291;
else if (News1=="Nationwide House Prices m/m") r1=292;
else if (News1=="Net Lending to Individuals m/m") r1=293;
else if (News1=="New Home Sales") r1=294;
else if (News1=="New Home Sales m/m") r1=295;
else if (News1=="New House Price Index m/m") r1=296;
else if (News1=="New Housing Price Index m/m") r1=297;
else if (News1=="New Motor Vehicle Sales") r1=298;
else if (News1=="New Motor Vehicle Sales m/m") r1=299;
else if (News1=="New Vehicle Sales m/m") r1=300;
else if (News1=="New York Fed Geithner Speaks") r1=301;
else if (News1=="New York Fed Pres. Geithner Speaks") r1=302;
else if (News1=="New York Fed President Geithner Speaks") r1=303;
else if (News1=="Nonfarm Employment Change") r1=304;
else if (News1=="Nonfarm Productivity 2Q") r1=305;
else if (News1=="Nonfarm Productivity 2Q (f)") r1=306;
else if (News1=="Nonfarm Productivity q/q") r1=307;
else if (News1=="Nonfarm Productivity q/q (f)") r1=308;
else if (News1=="OECD Leading Indicators") r1=309;
else if (News1=="OPEC Press Conference") r1=310;
else if (News1=="Overall Household Spending y/y") r1=311;
else if (News1=="PCE Core m/m") r1=312;
else if (News1=="PCE Core y/y") r1=313;
else if (News1=="PCE Deflator y/y") r1=314;
else if (News1=="PCE Price Index m/m") r1=315;
else if (News1=="PPI Input m/m") r1=316;
else if (News1=="PPI Input q/q") r1=317;
else if (News1=="PPI Output m/m") r1=318;
else if (News1=="PPI Output q/q") r1=319;
else if (News1=="PPI excl. Food and Energy m/m") r1=320;
else if (News1=="PPI m/m") r1=321;
else if (News1=="PPI q/q") r1=322;
else if (News1=="Personal Income") r1=323;
else if (News1=="Personal Income m/m") r1=324;
else if (News1=="Personal Income y/y") r1=325;
else if (News1=="Personal Spending") r1=326;
else if (News1=="Personal Spending m/m") r1=327;
else if (News1=="Philadelphia Fed Index") r1=328;
else if (News1=="Philadelphia Fed Manufacturing Index") r1=329;
else if (News1=="Philadelphia Fed Pres. Plosser Speaks") r1=330;
else if (News1=="Philly Fed President Plosser Speaks") r1=331;
else if (News1=="Philly Fed President Santomero Speaks") r1=332;
else if (News1=="Pres. Bush State of the Union Address") r1=333;
else if (News1=="President Bush Speaks") r1=334;
else if (News1=="Producer Price Index Input m/m") r1=335;
else if (News1=="Producer Price Index Output m/m") r1=336;
else if (News1=="Producer Price Index m/m") r1=337;
else if (News1=="Producer Price Index m/m (Input)") r1=338;
else if (News1=="Producer Price Index m/m (Output)") r1=339;
else if (News1=="Producer Price Index q/q") r1=340;
else if (News1=="Public Sector Net Borrowing") r1=341;
else if (News1=="RBA Assist. Governor Battellino Speaks") r1=342;
else if (News1=="RBA Bulletin") r1=343;
else if (News1=="RBA Economist Richards Speaks") r1=344;
else if (News1=="RBA Governor Macfarlane Speaks") r1=345;
else if (News1=="RBA Governor Stevens Speaks") r1=346;
else if (News1=="RBA Interest Rate Statement") r1=347;
else if (News1=="RBA Quarterly Statement") r1=348;
else if (News1=="RBNZ Governor Bollard Speaks") r1=349;
else if (News1=="RBNZ Interest Rate Statement") r1=350;
else if (News1=="RICS House Price Balance") r1=351;
else if (News1=="RMPI m/m") r1=352;
else if (News1=="RPI m/m") r1=353;
else if (News1=="Raw Materials Price Index m/m") r1=354;
else if (News1=="Retail PMI") r1=355;
else if (News1=="Retail Price Index m/m") r1=356;
else if (News1=="Retail Sales excl. Autos m/m") r1=357;
else if (News1=="Retail Sales m/m") r1=358;
else if (News1=="Retail Sales m/m (p)") r1=359;
else if (News1=="Retail Sales y/y") r1=360;
else if (News1=="Richmond Fed Index") r1=361;
else if (News1=="Richmond Fed Pres. Lacker Speaks") r1=362;
else if (News1=="Richmond Fed President Lacker Speaks") r1=363;
else if (News1=="Rightmove House Price Index m/m") r1=364;
else if (News1=="SECO Economic Forecasts") r1=365;
else if (News1=="SNB 2006 Media News Conference") r1=366;
else if (News1=="SNB President Roth Speaks") r1=367;
else if (News1=="SVME PMI") r1=368;
else if (News1=="San Fran Fed President Yellen Speak") r1=369;
else if (News1=="San Fran Fed President Yellen Speaks") r1=370;
else if (News1=="Services PMI") r1=371;
else if (News1=="Small Business Confidence") r1=372;
else if (News1=="St. Louis Fed Pres. Poole Speaks") r1=373;
else if (News1=="St. Louis Fed President Poole Speaks") r1=374;
else if (News1=="TDMI Inflation Gauge m/m") r1=375;
else if (News1=="TIC Report") r1=376;
else if (News1=="Tankan Large Manufacturers Index") r1=377;
else if (News1=="Tankan Large Manufacturers Index q/q") r1=378;
else if (News1=="Tankan Lg. Manufacturers Index") r1=379;
else if (News1=="Tankan Lg. Non-Manufacturers Index") r1=380;
else if (News1=="Tankan Non-Manufacturers Index") r1=381;
else if (News1=="Tankan Non-Manufacturers Index q/q") r1=382;
else if (News1=="Tertiary Industry Activity Index m/m") r1=383;
else if (News1=="Tertiary Industry Index m/m") r1=384;
else if (News1=="Tokyo CPI excl. Fresh Food m/m") r1=385;
else if (News1=="Tokyo CPI m/m") r1=386;
else if (News1=="Tokyo Consumer Price Index m/m") r1=387;
else if (News1=="Total Car Production q/q") r1=388;
else if (News1=="Trade Balance") r1=389;
else if (News1=="Trade Balance (f)") r1=390;
else if (News1=="Trade Balance (p)") r1=391;
else if (News1=="Treasury Budget") r1=392;
else if (News1=="Treasury Secretary Adams Testifies") r1=393;
else if (News1=="Treasury Secretary Paulson Speaks") r1=394;
else if (News1=="US Treasury Snow Senate Testimony") r1=395;
else if (News1=="Unemployment Claims") r1=396;
else if (News1=="Unemployment Rate") r1=397;
else if (News1=="Unit Labor Costs 2Q") r1=398;
else if (News1=="Unit Labor Costs 2Q (f)") r1=399;
else if (News1=="Unit Labor Costs q/q") r1=400;
else if (News1=="Unit Labor Costs q/q (f)") r1=401;
else if (News1=="Vehicle Sales y/y") r1=402;
else if (News1=="Visitor Arrivals") r1=403;
else if (News1=="WMI Consumer Sentiment m/m") r1=404;
else if (News1=="WPI q/q") r1=405;
else if (News1=="Wage Price Index q/q") r1=406;
else if (News1=="Wholesale Inventories") r1=407;
else if (News1=="Wholesale Inventories m/m") r1=408;
else if (News1=="Wholesale Sales m/m") r1=409;
else if (News1=="Workers\' Household Spending m/m") r1=410;
else if (News1=="ZEW Economic Sentiment") r1=411;
  return (r1);
}

